@@ -90,10 +90,11 @@ typedef unsigned long long  uint64_t;
 #ifndef __MINGW32__
 #ifdef  _WIN64
 typedef __int64             ssize_t;
-#else
+#else /* _WIN64 */
 typedef int                 ssize_t;
-#endif
-#endif
+#endif /* _WIN64 */
+#endif /* !__MINGW32__ */
+#define PATH_MAX MAX_PATH
 #define USE_IPv6
 #define _CRT_SECURE_NO_DEPRECATE
 #define _CRT_NONSTDC_NO_DEPRECATE
