@@ -78,7 +78,7 @@ NOEXPORT void *cron_thread(void *arg) {
     struct sched_param param;
 #endif
 
-    (void)arg; /* skip warning about unused parameter */
+    (void)arg; /* squash the unused parameter warning */
     tls_alloc(NULL, NULL, "cron");
 #ifdef SCHED_BATCH
     param.sched_priority=0;
@@ -98,7 +98,7 @@ int cron_init() {
 }
 
 NOEXPORT void cron_thread(void *arg) {
-    (void)arg; /* skip warning about unused parameter */
+    (void)arg; /* squash the unused parameter warning */
     tls_alloc(NULL, NULL, "cron");
     if(!SetThreadPriority(GetCurrentThread(), THREAD_PRIORITY_LOWEST))
         ioerror("SetThreadPriority");
@@ -124,7 +124,7 @@ NOEXPORT void cron_worker(void) {
     time_t now, then;
     int delay;
 
-    s_log(LOG_DEBUG, "Cron started");
+    s_log(LOG_DEBUG, "Cron thread initialized");
     sleep(60); /* allow the other services to start with idle CPU */
     time(&then);
     for(;;) {
@@ -183,10 +183,10 @@ NOEXPORT void cron_dh_param(void) {
 #endif
 
     /* update global dh_params for future configuration reloads */
-    enter_critical_section(CRIT_DH); /* it only needs an rwlock here */
+    CRYPTO_w_lock(stunnel_locks[LOCK_DH]);
     DH_free(dh_params);
     dh_params=dh;
-    leave_critical_section(CRIT_DH);
+    CRYPTO_w_unlock(stunnel_locks[LOCK_DH]);
 
     /* set for all sections that require it */
     for(opt=service_options.next; opt; opt=opt->next)
